@@ -35,22 +35,18 @@ def _is_posix():
 # ==============================================================================
 
 
-def _is_os_system_call(node):
+def _is_function_call(node, module, function):
+    if not isinstance(function, (list, tuple)):
+        function = (function,)
     return (
         isinstance(node.func, astroid.Attribute)
         and isinstance(node.func.expr, astroid.Name)
-        and node.func.expr.name == 'os'
-        and node.func.attrname == 'system'
+        and node.func.expr.name == module
+        and node.func.attrname in function
     )
 
 
-def _is_os_popen_call(node):
-    return (
-        isinstance(node.func, astroid.Attribute)
-        and isinstance(node.func.expr, astroid.Name)
-        and node.func.expr.name == 'os'
-        and node.func.attrname == 'popen'
-    )
+# ------------------------------------------------------------------------------
 
 
 def _is_os_path_call(node):
@@ -95,6 +91,32 @@ def _is_builtin_open_for_writing(node):
     return False
 
 
+def _is_os_open_allowed_mode(node, allowed_modes):
+    mode = None
+    flags = None  # pylint: disable=unused-variable
+    if len(node.args) > 1 and isinstance(node.args[1], (astroid.Attribute, astroid.BinOp)):
+        # Cover:
+        #  * os.open(xxx, os.O_WRONLY)
+        #  * os.open(xxx, os.O_WRONLY | os.O_CREATE)
+        #  * os.open(xxx, os.O_WRONLY | os.O_CREATE | os.O_FSYNC)
+        flags = node.args[1]
+    if len(node.args) > 2 and isinstance(node.args[2], astroid.Const):
+        mode = node.args[2].value
+    elif node.keywords:
+        for keyword in node.keywords:
+            if keyword.arg == 'flags':
+                flags = keyword.value  # pylint: disable=unused-variable # noqa: F841
+            if keyword.arg == 'mode':
+                mode = keyword.value.value
+                break
+    if mode is not None:
+        # TODO: condition check on the flags value if present (ie. ignore if read-only)
+        return mode in allowed_modes
+
+    # NB: default to True in all other cases
+    return True
+
+
 def _is_shell_true_call(node):
     if not (isinstance(node.func, astroid.Attribute) and isinstance(node.func.expr, astroid.Name)):
         return False
@@ -200,35 +222,30 @@ def _is_yaml_unsafe_call(node):
     return False
 
 
-def _is_jsonpickle_encode_call(node):
-    if isinstance(node.func, astroid.Attribute):
-        if (
-            isinstance(node.func.expr, astroid.Name)
-            and node.func.expr.name == 'jsonpickle'
-            and node.func.attrname == 'decode'
-        ):
-            return True
-    return False
-
-
-def _is_shlex_quote_call(node):
-    return not _is_posix() and (
-        isinstance(node.func, astroid.Attribute)
-        and isinstance(node.func.expr, astroid.Name)
-        and node.func.expr.name == 'shlex'
-        and node.func.attrname == 'quote'
-    )
-
-
 # ==============================================================================
 
 
 class SecureCodingStandardChecker(BaseChecker):
     """Plugin class."""
 
-    __implements__ = IAstroidChecker
+    DEFAULT_MAX_MODE = 0o755
+    W8012_DISPLAY_MSG = 'Avoid using `os.open` with unsafe permissions permissions'
+
+    __implements__ = (IAstroidChecker,)
 
     name = 'secure-coding-standard'
+    options = (
+        (
+            'os-open-mode',
+            {
+                'default': False,
+                'type': 'string',
+                'metavar': '<os-open-mode>',
+                'help': 'Integer or comma-separated list of integers (octal or decimal) of allowed modes. If set to a '
+                'truthful value (ie. >0 or non-empty list), this checker will prefer `os.open` over the builtin `open`',
+            },
+        ),
+    )
     priority = -1
 
     msg = {
@@ -294,11 +311,35 @@ class SecureCodingStandardChecker(BaseChecker):
             'avoid-shlex-quote-on-non-posix',
             'Use of `shlex.quote()` should be avoided on non-POSIX platforms (such as Windows)',
         ),
+        'W8012': (
+            W8012_DISPLAY_MSG,
+            'os-open-unsafe-permissions',
+            'Avoid using `os.open` with unsafe file permissions (by default 0 <= mode <= 0o755)',
+        ),
+        'E8013': (
+            'Avoid using `pickle.load` and `pickle.loads`',
+            'avoid-pickle-load',
+            'Use of `pickle.load` and `pickle.loads` should be avoided in favour of safer file formats',
+        ),
+        'E8014': (
+            'Avoid using `marshal.load` and `marshal.loads`',
+            'avoid-marshal-load',
+            'Use of `marshal.load` and `marshal.loads` should be avoided in favour of safer file formats',
+        ),
+        'E8015': (
+            'Avoid using `shelve.open()`',
+            'avoid-shelve-open',
+            'Use of `shelve.open()` should be avoided in favour of safer file formats',
+        ),
     }
 
-    options = {}
+    def __init__(self, *args, **kwargs):
+        """Initialize a SecureCodingStandardChecker object."""
+        super().__init__(*args, **kwargs)
+        self._prefer_os_open = False
+        self._os_open_modes_allowed = []
 
-    def visit_call(self, node):
+    def visit_call(self, node):  # pylint: disable=too-many-branches
         """Visitor method called for astroid.Call nodes."""
         if _is_pdb_call(node):
             self.add_message('avoid-debug-stmt', node=node)
@@ -306,22 +347,34 @@ class SecureCodingStandardChecker(BaseChecker):
             self.add_message('replace-mktemp', node=node)
         elif _is_yaml_unsafe_call(node):
             self.add_message('avoid-yaml-unsafe-load', node=node)
-        elif _is_jsonpickle_encode_call(node):
+        elif _is_function_call(node, module='jsonpickle', function='decode'):
             self.add_message('avoid-jsonpickle-decode', node=node)
-        elif _is_os_system_call(node):
+        elif _is_function_call(node, module='os', function='system'):
             self.add_message('avoid-os-system', node=node)
         elif _is_os_path_call(node):
             self.add_message('replace-os-relpath-abspath', node=node)
         elif _is_shell_true_call(node):
             self.add_message('avoid-shell-true', node=node)
-        elif _is_os_popen_call(node):
+        elif _is_function_call(node, module='os', function='popen'):
             self.add_message('avoid-os-popen', node=node)
-        elif _is_builtin_open_for_writing(node):
+        elif _is_builtin_open_for_writing(node) and self._prefer_os_open:
             self.add_message('replace-builtin-open', node=node)
         elif isinstance(node.func, astroid.Name) and (node.func.name in ('eval', 'exec')):
             self.add_message('avoid-eval-exec', node=node)
-        elif _is_shlex_quote_call(node):
+        elif not _is_posix() and _is_function_call(node, module='shlex', function='quote'):
             self.add_message('avoid-shlex-quote-on-non-posix', node=node)
+        elif (
+            _is_function_call(node, module='os', function='open')
+            and self._prefer_os_open
+            and not _is_os_open_allowed_mode(node, self._os_open_modes_allowed)
+        ):
+            self.add_message('os-open-unsafe-permissions', node=node)
+        elif _is_function_call(node, module='pickle', function=('load', 'loads')):
+            self.add_message('avoid-pickle-load', node=node)
+        elif _is_function_call(node, module='marshal', function=('load', 'loads')):
+            self.add_message('avoid-marshal-load', node=node)
+        elif _is_function_call(node, module='shelve', function='open'):
+            self.add_message('avoid-shelve-open', node=node)
 
     def visit_import(self, node):
         """Visitor method called for astroid.Import nodes."""
@@ -351,18 +404,100 @@ class SecureCodingStandardChecker(BaseChecker):
             self.add_message('avoid-os-popen', node=node)
         elif not _is_posix() and node.modname == 'shlex' and [name for (name, _) in node.names if name == 'quote']:
             self.add_message('avoid-shlex-quote-on-non-posix', node=node)
+        elif node.modname == 'pickle' and [name for (name, _) in node.names if name in ('load', 'loads')]:
+            self.add_message('avoid-pickle-load', node=node)
+        elif node.modname == 'marshal' and [name for (name, _) in node.names if name in ('load', 'loads')]:
+            self.add_message('avoid-marshal-load', node=node)
+        elif node.modname == 'shelve' and [name for (name, _) in node.names if name == 'open']:
+            self.add_message('avoid-shelve-open', node=node)
 
     def visit_with(self, node):
         """Visitor method called for astroid.With nodes."""
         for item in node.items:
-            if item and isinstance(item[0], astroid.Call) and _is_builtin_open_for_writing(item[0]):
-                self.add_message('replace-builtin-open', node=node)
+            if item and isinstance(item[0], astroid.Call):
+                if self._prefer_os_open:
+                    if _is_builtin_open_for_writing(item[0]):
+                        self.add_message('replace-builtin-open', node=node)
+                    elif _is_function_call(item[0], module='os', function='open') and not _is_os_open_allowed_mode(
+                        item[0], self._os_open_modes_allowed
+                    ):
+                        self.add_message('os-open-unsafe-permissions', node=node)
+                elif _is_function_call(item[0], module='shelve', function='open'):
+                    self.add_message('avoid-shelve-open', node=node)
 
     def visit_assert(self, node):
         """Visitor method called for astroid.Assert nodes."""
         self.add_message('avoid-assert', node=node)
 
+    def set_os_open_mode(self, arg):
+        """
+        Control whether we prefer `os.open` over the builtin `open`.
+
+        Args:
+            arg (str): String with with mode value. Can be either of:
+                - 'yes', 'y', 'true' (case-insensitive)
+                    The maximum mode value is then set to self.DEFAULT_MAX_MODE
+                - a single octal or decimal integer
+                    The maximum mode value is then set to that integer value
+                - a comma-separated list of integers (octal or decimal)
+                    The allowed mode values are then those found in the list
+                - anything else will disable the feature
+        """
+
+        def _str_to_int(arg):
+            try:
+                return int(arg, 8)
+            except ValueError:
+                return int(arg)
+
+        def _update_display_msg(suffix=''):
+            self.msg['W8012'] = (self.W8012_DISPLAY_MSG + suffix, self.msg['W8012'][1], self.msg['W8012'][2])
+
+        arg = arg.lower()
+        modes = [mode.strip() for mode in arg.split(',')]
+
+        if len(modes) > 1:
+            # Lists of allowed modes
+            try:
+                self._os_open_modes_allowed = [_str_to_int(mode) for mode in modes if mode]
+                if not self._os_open_modes_allowed:
+                    raise ValueError('Calculated empty value for `os_open_mode`!')
+                self._prefer_os_open = True
+                _update_display_msg(suffix=f' (mode in {modes})')
+            except ValueError as error:
+                raise ValueError(f'Unable to convert {modes} elements to integers!') from error
+        elif modes and modes[0]:
+            # Single values (ie. max allowed value for mode)
+            try:
+                val = _str_to_int(arg)
+                self._prefer_os_open = val > 0
+                if self._prefer_os_open:
+                    self._os_open_modes_allowed = list(range(0, val + 1))
+                    _update_display_msg(suffix=f' (mode <= {arg})')
+                else:
+                    self._os_open_modes_allowed.clear()
+            except ValueError as error:
+                if arg in ('y', 'yes', 'true'):
+                    self._prefer_os_open = True
+                    self._os_open_modes_allowed = list(range(0, self.DEFAULT_MAX_MODE + 1))
+                    _update_display_msg(suffix=f' (mode <= {oct(self.DEFAULT_MAX_MODE)})')
+                elif arg in ('n', 'no', 'false'):
+                    self._prefer_os_open = False
+                    self._os_open_modes_allowed.clear()
+                else:
+                    raise ValueError(f'Invalid value for `os_open_mode`: {arg}!') from error
+        else:
+            raise ValueError(f'Invalid value for `os_open_mode`: {arg}!')
+
 
 def register(linter):  # pragma: no cover
     """Register the plugin to Pylint."""
     linter.register_checker(SecureCodingStandardChecker(linter))
+
+
+def load_configuration(linter):  # pragma: no cover
+    """Load data from the configuration file."""
+    for checker in linter.get_checkers():
+        if isinstance(checker, SecureCodingStandardChecker):
+            checker.set_os_open_mode(checker.config.os_open_mode)
+            break
